@@ -14,17 +14,17 @@
 //! loader by default.
 
 use byteorder::{BigEndian, ReadBytesExt};
-use freetype::freetype::{FT_Byte, FT_Done_Face, FT_Error, FT_Face, FT_FACE_FLAG_FIXED_WIDTH};
-use freetype::freetype::{
-    FT_Fixed, FT_Get_Char_Index, FT_Get_Name_Index, FT_Get_Postscript_Name, FT_Pos,
+use freetype_sys::{
+    ft_sfnt_os2, FT_Byte, FT_Done_Face, FT_Done_FreeType, FT_Error, FT_Face, FT_Fixed,
+    FT_Get_Char_Index, FT_Get_Name_Index, FT_Get_Postscript_Name, FT_Get_Sfnt_Name,
+    FT_Get_Sfnt_Name_Count, FT_Get_Sfnt_Table, FT_Init_FreeType, FT_Library,
+    FT_Library_SetLcdFilter, FT_Load_Glyph, FT_Long, FT_Matrix, FT_New_Memory_Face, FT_Pos,
+    FT_Reference_Face, FT_Set_Char_Size, FT_Set_Transform, FT_UInt, FT_ULong, FT_Vector,
+    FT_FACE_FLAG_FIXED_WIDTH, FT_LCD_FILTER_DEFAULT, FT_LOAD_DEFAULT, FT_LOAD_MONOCHROME,
+    FT_LOAD_NO_HINTING, FT_LOAD_RENDER, FT_LOAD_TARGET_LCD, FT_LOAD_TARGET_LIGHT,
+    FT_LOAD_TARGET_MONO, FT_LOAD_TARGET_NORMAL, FT_PIXEL_MODE_GRAY, FT_PIXEL_MODE_LCD,
+    FT_PIXEL_MODE_LCD_V, FT_PIXEL_MODE_MONO, FT_STYLE_FLAG_ITALIC, TT_OS2,
 };
-use freetype::freetype::{FT_Get_Sfnt_Table, FT_Init_FreeType, FT_LcdFilter, FT_Library};
-use freetype::freetype::{FT_Library_SetLcdFilter, FT_Load_Glyph, FT_LOAD_DEFAULT};
-use freetype::freetype::{FT_Load_Sfnt_Table, FT_Long, FT_Matrix, FT_New_Memory_Face};
-use freetype::freetype::{FT_Reference_Face, FT_Set_Char_Size, FT_Set_Transform, FT_Sfnt_Tag};
-use freetype::freetype::{FT_UInt, FT_ULong, FT_UShort, FT_Vector, FT_STYLE_FLAG_ITALIC};
-use freetype::freetype::{FT_LOAD_MONOCHROME, FT_LOAD_NO_HINTING, FT_LOAD_RENDER};
-use freetype::tt_os2::TT_OS2;
 use log::warn;
 use pathfinder_geometry::line_segment::LineSegment2F;
 use pathfinder_geometry::rect::{RectF, RectI};
@@ -66,21 +66,6 @@ const TT_PLATFORM_APPLE_UNICODE: u16 = 0;
 const FT_POINT_TAG_ON_CURVE: c_char = 0x01;
 const FT_POINT_TAG_CUBIC_CONTROL: c_char = 0x02;
 
-const FT_RENDER_MODE_NORMAL: u32 = 0;
-const FT_RENDER_MODE_LIGHT: u32 = 1;
-const FT_RENDER_MODE_MONO: u32 = 2;
-const FT_RENDER_MODE_LCD: u32 = 3;
-
-const FT_LOAD_TARGET_LIGHT: u32 = (FT_RENDER_MODE_LIGHT & 15) << 16;
-const FT_LOAD_TARGET_LCD: u32 = (FT_RENDER_MODE_LCD & 15) << 16;
-const FT_LOAD_TARGET_MONO: u32 = (FT_RENDER_MODE_MONO & 15) << 16;
-const FT_LOAD_TARGET_NORMAL: u32 = (FT_RENDER_MODE_NORMAL & 15) << 16;
-
-const FT_PIXEL_MODE_MONO: u8 = 1;
-const FT_PIXEL_MODE_GRAY: u8 = 2;
-const FT_PIXEL_MODE_LCD: u8 = 5;
-const FT_PIXEL_MODE_LCD_V: u8 = 6;
-
 const OS2_FS_SELECTION_OBLIQUE: u16 = 1 << 9;
 
 // Not in our FreeType bindings, so we define these ourselves.
@@ -94,16 +79,29 @@ const BDF_PROPERTY_TYPE_INTEGER: BDF_PropertyType = 2;
 const BDF_PROPERTY_TYPE_CARDINAL: BDF_PropertyType = 3;
 
 thread_local! {
-    static FREETYPE_LIBRARY: FT_Library = {
+    static FREETYPE_LIBRARY: FtLibrary = {
         unsafe {
             let mut library = ptr::null_mut();
             assert_eq!(FT_Init_FreeType(&mut library), 0);
-            FT_Library_SetLcdFilter(library, FT_LcdFilter::FT_LCD_FILTER_DEFAULT);
-            library
+            FT_Library_SetLcdFilter(library, FT_LCD_FILTER_DEFAULT);
+            FtLibrary(library)
         }
     };
 }
 
+#[repr(transparent)]
+struct FtLibrary(FT_Library);
+
+impl Drop for FtLibrary {
+    fn drop(&mut self) {
+        unsafe {
+            let mut library = ptr::null_mut();
+            mem::swap(&mut library, &mut self.0);
+            FT_Done_FreeType(library);
+        }
+    }
+}
+
 /// The handle that the FreeType API natively uses to represent a font.
 pub type NativeFont = FT_Face;
 
@@ -137,7 +135,7 @@ impl Font {
         FREETYPE_LIBRARY.with(|freetype_library| unsafe {
             let mut freetype_face = ptr::null_mut();
             if FT_New_Memory_Face(
-                *freetype_library,
+                freetype_library.0,
                 (*font_data).as_ptr(),
                 font_data.len() as FT_Long,
                 font_index as FT_Long,
@@ -190,7 +188,7 @@ impl Font {
         loop {
             font_data.extend(iter::repeat(0).take(CHUNK_SIZE));
             let freetype_stream = (*freetype_face).stream;
-            let n_read = ((*freetype_stream).read.unwrap())(
+            let n_read = ((*freetype_stream).read)(
                 freetype_stream,
                 font_data.len() as FT_ULong,
                 font_data.as_mut_ptr(),
@@ -216,7 +214,7 @@ impl Font {
         FREETYPE_LIBRARY.with(|freetype_library| unsafe {
             let mut freetype_face = ptr::null_mut();
             if FT_New_Memory_Face(
-                *freetype_library,
+                freetype_library.0,
                 (*font_data).as_ptr(),
                 font_data.len() as FT_Long,
                 0,
@@ -244,7 +242,7 @@ impl Font {
 
             let mut freetype_face = ptr::null_mut();
             if FT_New_Memory_Face(
-                *freetype_library,
+                freetype_library.0,
                 (*font_data).as_ptr(),
                 font_data.len() as FT_Long,
                 0,
@@ -400,7 +398,7 @@ impl Font {
                 unsafe { FT_Get_Name_Index(self.freetype_face, ffi_name.as_ptr() as *mut c_char) };
 
             if code > 0 {
-                return Some(u32::from(code));
+                return Some(code);
             }
         }
         None
@@ -443,16 +441,17 @@ impl Font {
                 );
             }
 
-            if FT_Load_Glyph(self.freetype_face, glyph_id, load_flags as i32) != 0 {
+            if FT_Load_Glyph(self.freetype_face, glyph_id, load_flags) != 0 {
                 return Err(GlyphLoadingError::NoSuchGlyph);
             }
 
             let outline = &(*(*self.freetype_face).glyph).outline;
-            let contours =
-                slice::from_raw_parts((*outline).contours, (*outline).n_contours as usize);
-            let point_positions =
-                slice::from_raw_parts((*outline).points, (*outline).n_points as usize);
-            let point_tags = slice::from_raw_parts((*outline).tags, (*outline).n_points as usize);
+            if outline.n_contours == 0 {
+                return Ok(());
+            }
+            let contours = slice::from_raw_parts(outline.contours, outline.n_contours as usize);
+            let point_positions = slice::from_raw_parts(outline.points, outline.n_points as usize);
+            let point_tags = slice::from_raw_parts(outline.tags, outline.n_points as usize);
 
             let mut current_point_index = 0;
             for &last_point_index_in_contour in contours {
@@ -556,7 +555,7 @@ impl Font {
             }
 
             if hinting.grid_fitting_size().is_some() {
-                reset_freetype_face_char_size((*self).freetype_face)
+                reset_freetype_face_char_size(self.freetype_face)
             }
         }
 
@@ -591,7 +590,7 @@ impl Font {
             if FT_Load_Glyph(
                 self.freetype_face,
                 glyph_id,
-                (FT_LOAD_DEFAULT | FT_LOAD_NO_HINTING) as i32,
+                FT_LOAD_DEFAULT | FT_LOAD_NO_HINTING,
             ) != 0
             {
                 return Err(GlyphLoadingError::NoSuchGlyph);
@@ -616,7 +615,7 @@ impl Font {
             if FT_Load_Glyph(
                 self.freetype_face,
                 glyph_id,
-                (FT_LOAD_DEFAULT | FT_LOAD_NO_HINTING) as i32,
+                FT_LOAD_DEFAULT | FT_LOAD_NO_HINTING,
             ) != 0
             {
                 return Err(GlyphLoadingError::NoSuchGlyph);
@@ -669,7 +668,7 @@ impl Font {
 
     /// Returns true if and only if the font loader can perform hinting in the requested way.
     ///
-    /// Some APIs support only rasterizing glyphs with hinting, not retriving hinted outlines. If
+    /// Some APIs support only rasterizing glyphs with hinting, not retrieving hinted outlines. If
     /// `for_rasterization` is false, this function returns true if and only if the loader supports
     /// retrieval of hinted *outlines*. If `for_rasterization` is true, this function returns true
     /// if and only if the loader supports *rasterizing* hinted glyphs.
@@ -747,7 +746,7 @@ impl Font {
 
     fn get_os2_table(&self) -> Option<*const TT_OS2> {
         unsafe {
-            let table = FT_Get_Sfnt_Table(self.freetype_face, FT_Sfnt_Tag::FT_SFNT_OS2);
+            let table = FT_Get_Sfnt_Table(self.freetype_face, ft_sfnt_os2);
             if table.is_null() {
                 None
             } else {
@@ -830,7 +829,7 @@ impl Font {
                 hinting_options,
                 rasterization_options,
             );
-            if FT_Load_Glyph(self.freetype_face, glyph_id, load_flags as i32) != 0 {
+            if FT_Load_Glyph(self.freetype_face, glyph_id, load_flags) != 0 {
                 return Err(GlyphLoadingError::NoSuchGlyph);
             }
 
@@ -838,30 +837,43 @@ impl Font {
             // need to keep this around for bilevel rendering, as the direct API doesn't work with
             // that mode.
             let bitmap = &(*(*self.freetype_face).glyph).bitmap;
-            let bitmap_stride = (*bitmap).pitch as usize;
-            let bitmap_width = (*bitmap).width as i32;
-            let bitmap_height = (*bitmap).rows as i32;
+            let bitmap_stride = bitmap.pitch as usize;
+            let bitmap_width = bitmap.width;
+            let bitmap_height = bitmap.rows;
             let bitmap_size = Vector2I::new(bitmap_width, bitmap_height);
-            let bitmap_buffer = (*bitmap).buffer as *const i8 as *const u8;
+            let bitmap_buffer = bitmap.buffer as *const i8 as *const u8;
             let bitmap_length = bitmap_stride * bitmap_height as usize;
-            let buffer = slice::from_raw_parts(bitmap_buffer, bitmap_length);
-            let dst_point = Vector2I::new(
-                (*(*self.freetype_face).glyph).bitmap_left,
-                -(*(*self.freetype_face).glyph).bitmap_top,
-            );
+            if bitmap_buffer.is_null() {
+                assert_eq!(
+                    bitmap_length, 0,
+                    "bitmap length should be 0 when bitmap_buffer is nullptr"
+                );
+            } else {
+                let buffer = slice::from_raw_parts(bitmap_buffer, bitmap_length);
+                let dst_point = Vector2I::new(
+                    (*(*self.freetype_face).glyph).bitmap_left,
+                    -(*(*self.freetype_face).glyph).bitmap_top,
+                );
 
-            // FIXME(pcwalton): This function should return a Result instead.
-            match (*bitmap).pixel_mode {
-                FT_PIXEL_MODE_GRAY => {
-                    canvas.blit_from(dst_point, buffer, bitmap_size, bitmap_stride, Format::A8);
+                // FIXME(pcwalton): This function should return a Result instead.
+                match bitmap.pixel_mode as u32 {
+                    FT_PIXEL_MODE_GRAY => {
+                        canvas.blit_from(dst_point, buffer, bitmap_size, bitmap_stride, Format::A8);
+                    }
+                    FT_PIXEL_MODE_LCD | FT_PIXEL_MODE_LCD_V => {
+                        canvas.blit_from(
+                            dst_point,
+                            buffer,
+                            bitmap_size,
+                            bitmap_stride,
+                            Format::Rgb24,
+                        );
+                    }
+                    FT_PIXEL_MODE_MONO => {
+                        canvas.blit_from_bitmap_1bpp(dst_point, buffer, bitmap_size, bitmap_stride);
+                    }
+                    _ => panic!("Unexpected FreeType pixel mode!"),
                 }
-                FT_PIXEL_MODE_LCD | FT_PIXEL_MODE_LCD_V => {
-                    canvas.blit_from(dst_point, buffer, bitmap_size, bitmap_stride, Format::Rgb24);
-                }
-                FT_PIXEL_MODE_MONO => {
-                    canvas.blit_from_bitmap_1bpp(dst_point, buffer, bitmap_size, bitmap_stride);
-                }
-                _ => panic!("Unexpected FreeType pixel mode!"),
             }
 
             FT_Set_Transform(self.freetype_face, ptr::null_mut(), ptr::null_mut());
@@ -874,7 +886,7 @@ impl Font {
         &self,
         hinting: HintingOptions,
         rasterization: RasterizationOptions,
-    ) -> u32 {
+    ) -> i32 {
         let mut options = match (hinting, rasterization) {
             (HintingOptions::VerticalSubpixel(_), _) | (_, RasterizationOptions::SubpixelAa) => {
                 FT_LOAD_TARGET_LCD
@@ -969,11 +981,14 @@ impl Clone for Font {
 
 impl Drop for Font {
     fn drop(&mut self) {
-        unsafe {
-            if !self.freetype_face.is_null() {
+        // The AccessError can be ignored, as it means FREETYPE_LIBRARY has already been
+        // destroyed, and it already destroys all FreeType resources.
+        // https://freetype.org/freetype2/docs/reference/ft2-module_management.html#ft_done_library
+        let _ = FREETYPE_LIBRARY.try_with(|freetype_library| unsafe {
+            if !freetype_library.0.is_null() && !self.freetype_face.is_null() {
                 assert_eq!(FT_Done_Face(self.freetype_face), 0);
             }
-        }
+        });
     }
 }
 
@@ -1150,16 +1165,6 @@ unsafe fn reset_freetype_face_char_size(face: FT_Face) {
     }
 }
 
-#[repr(C)]
-struct FT_SfntName {
-    platform_id: FT_UShort,
-    encoding_id: FT_UShort,
-    language_id: FT_UShort,
-    name_id: FT_UShort,
-    string: *mut FT_Byte,
-    string_len: FT_UInt,
-}
-
 trait F32ToFtFixed {
     type Output;
     fn f32_to_ft_fixed_26_6(self) -> Self::Output;
@@ -1216,16 +1221,21 @@ extern "C" {
         value: *mut c_void,
         value_len: FT_Long,
     ) -> FT_Long;
-    fn FT_Get_Sfnt_Name(face: FT_Face, idx: FT_UInt, aname: *mut FT_SfntName) -> FT_Error;
-    fn FT_Get_Sfnt_Name_Count(face: FT_Face) -> FT_UInt;
+    fn FT_Load_Sfnt_Table(
+        face: FT_Face,
+        tag: FT_ULong,
+        offset: FT_Long,
+        buffer: *mut FT_Byte,
+        length: *mut FT_ULong,
+    ) -> FT_Error;
 }
 
 #[cfg(test)]
 mod test {
     use crate::loaders::freetype::Font;
 
-    static PCF_FONT_PATH: &'static str = "resources/tests/times-roman-pcf/timR12.pcf";
-    static PCF_FONT_POSTSCRIPT_NAME: &'static str = "Times-Roman";
+    static PCF_FONT_PATH: &str = "resources/tests/times-roman-pcf/timR12.pcf";
+    static PCF_FONT_POSTSCRIPT_NAME: &str = "Times-Roman";
 
     #[test]
     fn get_pcf_postscript_name() {
